/**
 * Created by AlexanderC on 6/15/15.
 */

'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.ObjectToJoi = undefined;

var _Joi = require('./Helpers/Joi');

var _joi = require('joi');

var _joi2 = _interopRequireDefault(_joi);

var _InvalidSchemaException = require('./Exception/InvalidSchemaException');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

/**
 * Exporting simple object to joi
 */
class ObjectToJoi {
  /**
   * @param {Object} baseObject
   */
  constructor(baseObject) {
    this._baseObject = baseObject;
  }

  /**
   * @returns {Object}
   */
  get baseObject() {
    return this._baseObject;
  }

  /**
   * @returns {Object}
   */
  transform() {
    try {
      return ObjectToJoi._transform(this._baseObject);
    } catch (e) {
      throw new _InvalidSchemaException.InvalidSchemaException(this._baseObject, e);
    }
  }

  /**
   * @param {Object} obj
   * @returns {Object}
   * @private
   */
  static _transform(obj) {
    let transObj = {};

    for (let property in obj) {
      if (!obj.hasOwnProperty(property)) {
        continue;
      }

      let value = obj[property];

      if (typeof value === 'object') {
        transObj[property] = ObjectToJoi._transform(value);
      } else {
        let validationSchema = _Joi.Joi[value];

        if (typeof validationSchema === 'undefined') {
          throw new _InvalidSchemaException.InvalidSchemaException(obj, `Unknown field type ${value}`);
        }

        transObj[property] = validationSchema;
      }
    }

    return _joi2.default.object().keys(transObj);
  }
}
exports.ObjectToJoi = ObjectToJoi;